

use FTFile;
use Log;
use PeopleDB;
use RemoteDB;

use integer;

package EventDB;

my $init = 0;
my $loadedtree = 0;
my @eventlist;
my %keyedeventlist;

sub addEvent {

    my $_user = $_[0];
    my $_tree = $_[1];
    my $_nick = $_[2];
    my $_event = $_[3];
    my $_when = $_[4];
    my $_where = $_[5];
    my $_p1 = $_[6];
    my $_p2 = $_[7];
    my $_ref = $_[8];
    my $_comment = $_[9];

    if (PeopleDB::isKnown ($_user, $_tree, $_nick) &&
        ($_p1 !~ /.+/ || PeopleDB::isKnown ($_user, $_tree, $_p1)) &&
        ($_p2 !~ /.+/ || PeopleDB::isKnown ($_user, $_tree, $_p2))) { 

        open (EVENTS, ">>./trees/$_tree/events.db");

        if (($_event =~ /d/ || $_event =~ /M/) && $_p1 =~ /\w+/) {
            FTFile::write (\*EVENTS, "$_nick.$_p1.$_when.$_event", $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
            FTFile::write (\*EVENTS, "$_p1.$_nick.$_when.$_event", $_p1, $_event, $_when, $_nick, $_p2, $_where, $_tree, $_ref, $_comment);
        } else {
            FTFile::write (\*EVENTS, "$_nick.$_when.$_event", $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
        }

        close (EVENTS);

        Log::log ($_user, $_tree, "Added event type '$_event' ($_when $_where) for $_nick");

        return "";
    } else {
        return "Failed - non-existent person ($_nick, $_p1, $_p2)";
    }
}

sub replaceEvent {

    my $_user = $_[0];
    my $_tree = $_[1];
    my $_nick = $_[2];
    my $_event = $_[3];
    my $_when = $_[4];
    my $_where = $_[5];
    my $_p1 = $_[6];
    my $_p2 = $_[7];
    my $_ref = $_[8];
    my $_comment = $_[9];

    my $set = 0;

    open (EVENTS, "<./trees/$_tree/events.db");
    open (NEWEVENTS, ">./trees/$_tree/events.tmp.db");

    EVENT: while (<EVENTS>) {
        ($key, $n, $type) = split (/\|/);
        if (!$set && $n eq $_nick && $type eq $_event) {
            $set = 1;
            if ($_when =~ /\w+/) {
                FTFile::write (\*NEWEVENTS, "$_nick.$_when.$_event", $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
                next EVENT;
            }
        } else {
            print NEWEVENTS $_;
        }
    }

    if (!$set) {
        if ($_when =~ /\w+/) {
            FTFile::write (\*NEWEVENTS, "$_nick.$_when.$_event", $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
        }
    }

    close (NEWEVENTS);
    close (EVENTS);

    unlink "./trees/$_tree/events.db";
    rename "./trees/$_tree/events.tmp.db", "./trees/$_tree/events.db";

    Log::log ($_user, $_tree, "Replaced event type '$_event' ($_when $_where) for $_nick");
}


sub replaceEventByKey {

    my $_user = $_[0];
    my $_tree = $_[1];
    my $_nick = $_[2];
    my $_key = $_[3];
    my $_event = $_[4];
    my $_when = $_[5];
    my $_where = $_[6];
    my $_p1 = $_[7];
    my $_p2 = $_[8];
    my $_ref = $_[9];
    my $_comment = $_[10];

    (my $n, my $p1, my $d, my $t) = split (/\./, $_key);
    $altkey = "$p1.$n.$d.$t";

    if (($_event eq 'd' || $_event eq 'M') && $_p1 =~ /\w+/) {
        $newkey = "$_nick.$_p1.$_when.$_event";
        $newaltkey = "$_p1.$_nick.$_when.$_event";
    } else {
        $newkey = "$_nick.$_when.$_event";
    }

    my $set = 0;

    open (EVENTS, "<./trees/$_tree/events.db");
    open (NEWEVENTS, ">./trees/$_tree/events.tmp.db");

    EVENT: while (<EVENTS>) {
        ($key) = split (/\|/);

        if ($key eq $_key) {
            $set = 1;
            FTFile::write (\*NEWEVENTS, $newkey, $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
            next EVENT;
        } elsif ($key eq $altkey) {
            $set = 1;
            FTFile::write (\*NEWEVENTS, $newaltkey, $_p1, $_event, $_when, $_nick, $_p2, $_where, $_tree, $_ref, $_comment);
            next EVENT;
        } else {
            print NEWEVENTS $_;
        }
    }

    if (!$set) {
        if (($_event =~ /d/ || $_event =~ /M/) && $_p1 =~ /\w+/) {
            FTFile::write (\*NEWEVENTS, $newkey, $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
            FTFile::write (\*NEWEVENTS, $newaltkey, $_p1, $_event, $_when, $_nick, $_p2, $_where, $_tree, $_ref, $_comment);
        } else {
            FTFile::write (\*NEWEVENTS, $newkey, $_nick, $_event, $_when, $_p1, $_p2, $_where, $_tree, $_ref, $_comment);
        }
    }

    close (NEWEVENTS);
    close (EVENTS);

    unlink "./trees/$_tree/events.db";
    rename "./trees/$_tree/events.tmp.db", "./trees/$_tree/events.db";

    Log::log ($_user, $_tree, "Replaced event type '$_event' ($_when $_where) for $_nick");
}


sub eventDetailsByKey {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_key=$_[2];

    my $k;
    my $n;
    my $t;
    my $when;
    my $p1;
    my $p2;
    my $where;
    my $tr;
    my $ref;
    my $comment;

    open (EVENTS, "<./trees/$_tree/events.db");
    EVENT: while (<EVENTS>) {
        chomp;
        ($k) = split (/\|/);
        if ($k eq $_key) {
            ($k, $n, $t, $when, $p1, $p2, $where, $tr, $ref, $comment) = split (/\|/);
            last EVENT;
        }
    }
    close (EVENTS);

    return ($t, $when, $where, $p1, $p2, $ref, $comment);
}


sub deleteEventByKey {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_key=$_[2];

    (my $n, my $p1, my $d, my $t) = split (/\./, $_key);
    $altkey = "$p1.$n.$d.$t";

    open (EVENTS, "<./trees/$_tree/events.db");
    open (NEWEVENTS, ">./trees/$_tree/events.tmp.db");
    while (<EVENTS>) {
        chomp;
        (my $k) = split (/\|/);
        if ($k ne $_key && $k ne $altkey) {
            print NEWEVENTS "$_\n";
        }
    }
    close (NEWEVENTS);
    close (EVENTS);

    unlink "./trees/$_tree/events.db";
    rename "./trees/$_tree/events.tmp.db", "./trees/$_tree/events.db";

    Log::log ($_user, $_tree, "Deleted event key $_key");
}


sub deleteEvent {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_nick=$_[2];
    my $nickset= ($_nick =~ /\w+/);
    my $_date=$_[3];
    my $dateset= ($_date =~ /\w+/);
    my $_type=$_[4];
    my $typeset= ($_type =~ /\w+/);


    open (EVENTS, "<./trees/$_tree/events.db");
    open (NEWEVENTS, ">./trees/$_tree/events.tmp.db");
    EVENT: while (<EVENTS>) {
        chomp;
        (my $k, my $n, my $type, my $date, my $p1, my $p2, my $w, my $t) = split (/\|/);

        if ($t eq $tree) {

            if ((!$nickset || ($_nick ne $n && $_nick ne $p1 && $_nick ne $p2)) &&
                (!$dateset || $_date ne $date) &&
                (!$typeset || $_type ne $type)) {
                print NEWEVENTS "$_\n";
            } elsif ($nickset) {

                next EVENT if ($_nick eq $n);

                if ($_nick eq $_p1 && $_p2 =~ /\w+/) {
                    s/\|$_nick\|/\|\|/g;
                    print NEWEVENTS "$_\n";
                }
            }
        } else {
            print NEWEVENTS "$_\n";
        }
    }
    close (NEWEVENTS);
    close (EVENTS);

    unlink "./trees/$_tree/events.db";
    rename "./trees/$_tree/events.tmp.db", "./trees/$_tree/events.db";

    Log::log ($_user, $_tree, "Deleted event $_nick, $_date, $_type");
}



sub initialise {

    my $_tree = $_[0];
    my $_nick = $_[1];

    if ($_nick =~ /\w+/) {
        $_nick = LinksDB::resolve ($_tree, $_nick);

        if ($_nick =~ /\w+\.\w+/) {
            ($_tree, my $n) = split (/\./, $_nick);
        }
    }

    open (EVENTS, "<./trees/$_tree/events.db");

    my @fields = FTFile::read (\*EVENTS);
    my $done = 0;
    
    while (!$done) {

        (my $k, my $n, my $type, my $d, my $p1, my $p2, my $w, my $t) = @fields;

        $done = ($k !~ /.+/);

        if (!$done) {
            if ($t eq $_tree) {
                my $el = join ('|', @fields);
                push @eventlist, $el;
                $keyedeventlist{"$n.$type"} = $el;
            }

            @fields = FTFile::read (\*EVENTS);
        }
    }

    close (EVENTS);

    open (LINKS, "<./trees/$_tree/links.idx");

    while (<LINKS>) {
        chomp;
        (my $tt, my $tn, my $ot, my $on) = split (/\|/);

        if ($tt eq $_tree) {
            my @el = RemoteDB::eventsFor ($ot, $on);

            foreach $e (@el) {
                (my $k, my $n, my $type) = split (/\|/, $e);
                push @eventlist, $e;
                $keyedeventlist{"$ot.$on.$type"} = $e;
            }
        }
    }

    close (LINKS);

    $init = 1;
    $loadedtree = $_tree;
}


sub eventDate {

    my $_tree = $_[0];
    my $nick = $_[1];
    my $type = $_[2];

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree, $nick) ;
    }


    my $re = $keyedeventlist{"$nick.$type"};

    (my $k, my $n, my $t, my $when) = split (/\|/, $re);

    return $when;
}


sub eventDetails {

    my $_tree = $_[0];
    my $nick = $_[1];
    my $type = $_[2];

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree, $nick);
    }

    my $re = $keyedeventlist{"$nick.$type"};

    (my $k, my $n, my $t, my $when, my $p1, my $p2, my $where, my $t, my $ref, my $comment) = split (/\|/, $re);

    return ($when, $where, $p1, $p2, $ref, $comment);
}
 

sub eventDetailsFor {

    my $_tree = $_[0];
    my $nick = $_[1];

    if ($nick =~ /\w+\.\w+/) {
        (my $_tree, $nick) = split (/\./, $nick);
    }

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree, $nick);
    }
    my $nickset = ($nick =~ /\w+/);
    my $absnick;

    if ($nickset) {
        $absnick = LinksDB::fullyResolve($_tree, $nick);
    }

    my @res;

    foreach $re (@eventlist) {

        (my $k, my $n, my $t, my $when, my $p1, my $p2, my $where, my $tr,my $ref, my $comment) = split (/\|/, $re);

        if (!$nickset || ($nick eq $n || $p1 eq $nick || $p2 eq $nick) ||
           ($absnick eq $n || $absnick eq $p1 || $absnick eq $p2)) {
            push @res, [($k, $n, $t, $when, $where, $p1, $p2, $tr, $ref, $comment)];
        }
    }

    return @res;
}


sub allEventDetails {

    my $_tree = $_[0];

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree, "");
    }

    return (@eventlist);
}


sub eventTypes {
    my $_user = $_[0];
    my $_tree = $_[1];

    my @et;

    # code, 1st person, 3rd person, singular, howmany, mutual

    push @et, [('B', 'Born', 'Birth of', 'Birth of', 1, 1, 0)];
    push @et, [('A', 'Adopted by', 'Adoption of|by', 'Adopted', 0, 3, 0)];
    push @et, [('M', 'Marriage to', 'Marriage of', 'Marriage of', 0, 2, 1)];
    push @et, [('d', 'Divorce from', 'Divorce of', 'Divorce of', 0, 2, 1)];
    push @et, [('S', 'Moved to', 'Moved', 'Moved', 0, 1, 0)];
    push @et, [('T', 'Started at', 'Started', 'Started', 0, 1, 0)];
    push @et, [('F', 'Finished at', 'Finished', 'Finished', 0, 1, 0)];
    push @et, [('D', 'Died', 'Death of', 'Death of', 1, 1, 0)];

    return @et;
}


1

