

use Log;

use integer;
use RemoteDB;


package ParentsDB;

my $init = 0;
my %parentlist;

sub addParent {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];
    my $m = $_[3];
    my $f = $_[4];

    if ($m =~ /\w+/ || $f =~ /\w+/) {
        open (PARENTS, ">>./trees/$tree/parents.db");

        print PARENTS "$nick|$m|$f\n";

        close (PARENTS);
    }

    Log::log ($user, $tree, "Add parents ($m, $f) for $nick");

    return "";
}



sub initialise {

    $_tree = $_[0];

    open (PARENTS, "<./trees/$_tree/parents.db");

    while (<PARENTS>) {
        chomp;

        (my $n, my $m, my $f) = split (/\|/);

        $parentlist{$n} = "$m|$f";
    }

    close (PARENTS);


    open (LINKS, "<./trees/$_tree/links.idx");

    while (<LINKS>) {
        chomp;
        (my $tt, my $tn, my $ot, my $on) = split (/\|/);

        my @pc = RemoteDB::parentalConnections ($ot, $on);

        foreach $p (@pc) {
            $n = $p->[0];
            $m = $p->[1];
            $f = $p->[2];

            if ($n eq "$ot.$on") {
                $n = $tn;
            } elsif ($m eq "$ot.$on") {
                $m = $tn;
            } elsif ($f eq "$ot.$on") {
                $f = $tn;
            }

            $parentlist{$n} = "$m|$f";
        }
    }

    close (LINKS);

    $init = 1;
}


sub replaceParents {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];
    my $m = $_[3];
    my $f = $_[4];

    my $set = 0;

    open (PARENTS, "<./trees/$tree/parents.db");
    open (NEWPARENTS, ">./trees/$tree/parents.tmp.db");

    PERSON: while (<PARENTS>) {
        ($n) = split (/\|/);
        if (!$set && $n eq $nick) {
            $set = 1;
            if ($m =~ /\w+/ || $f =~ /\w+/) {
                print NEWPARENTS "$nick|$m|$f\n";
            }
            next PERSON;
        } else {
            print NEWPARENTS $_;
        }
    }

    if (!$set && ($m =~ /\w+/ || $f =~ /\w+/)) {
        print NEWPARENTS "$nick|$m|$f\n";
    }

    close (NEWPARENTS);
    close (PARENTS);

    unlink "./trees/$tree/parents.db";
    rename "./trees/$tree/parents.tmp.db", "./trees/$tree/parents.db";

    Log::log ($user, $tree, "Replace parents ($m, $f) for $nick");
}


sub parents {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];

    if ($nick =~ /\w+\.\w+/) {
        return RemoteDB::parents ($tree, $nick);
    }

    if (! $init) {
        initialise ($tree);
    }

    my $p = $parentlist{$nick};

    return split (/\|/, $p);
}


sub allParents {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];

    if ($nick =~ /\w+\.\w+/) {
        return RemoteDB::allParents ($tree, $nick);
    }

    if (! $init) {
        initialise ($tree);
    }

    my @pl;

    foreach my $k (keys %parentlist) {
        my @p = split (/\|/, $parentlist{$k});
        unshift (@p, ($k));
        push @pl, [@p];
    }
    return @pl;
}


sub children {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];

    if ($nick !~ /\w+/) {
        return;
    }

#    $nick = PeopleDB::qualify ($tree, $nick);

    if ($nick =~ /\w+\.\w+/) {
        return RemoteDB::children ($tree, $nick);
    }

    if (! $init) {
        initialise ($tree);
    }

    my @cl;
    my @p;

    foreach my $k (keys %parentlist) {
        @p = split (/\|/, $parentlist{$k});
        if ($p[0] eq $nick || $p[1] eq $nick) {
            push @cl, $k;
        }
    }

    return @cl;
}


sub deleteBothParents {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_key=$_[2];

    my $deleted = 0;

    open (PARENTS, "<./trees/$_tree/parents.db");
    open (NEWPARENTS, ">./trees/$_tree/parents.tmp.db");
    while (<PARENTS>) {
        (my $k) = split (/\|/);
        if ($k ne $_key) {
            print NEWPARENTS "$_";
        } else {
            $deleted = 1;
        }
    }
    close (NEWPARENTS);
    close (PARENTS);

    unlink "./trees/$_tree/parents.db";
    rename "./trees/$_tree/parents.tmp.db", "./trees/$_tree/parents.db";

    if ($init) {
        delete $parentlist{$_key};
    }

    if ($deleted) {
        Log::log ($_user, $_tree, "Delete parents for $_key");
    }
}


sub deleteAsParent {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_p=$_[2];

    open (PARENTS, "<./trees/$_tree/parents.db");
    open (NEWPARENTS, ">./trees/$_tree/parents.tmp.db");
    while (<PARENTS>) {
        chomp;
        (my $k, my $m, my $f) = split (/\|/);
        if ($_p eq $m) {
            $m = '';
        }
        if ($_p eq $f) {
            $f = '';
        }
        if ($m ne '' || $f ne '') {
             print NEWPARENTS "$k|$m|$f\n";
        }
    }
    close (NEWPARENTS);
    close (PARENTS);

    unlink "./trees/$_tree/parents.db";
    rename "./trees/$_tree/parents.tmp.db", "./trees/$_tree/parents.db";

    if ($init) {
        delete $parentlist{$_key};
    }

    Log::log ($_user, $_tree, "Delete $_p as a parent");
}

1
