#!/usr/bin/perl

use FTFile;
use Log;
use LinksDB;
use RemoteDB;

use integer;

package PeopleDB;

my $init = 0;
my $loadedtree;
my %peoplelist;

sub addPerson {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];
    my $familyname = $_[3];
    my $othernames = $_[4];
    my $sex = $_[5];
    my $homepage = $_[6];
    my $comment = $_[7];
    my $link = $_[8];

    open (PEOPLE, ">>./trees/$tree/people.idx");

    FTFile::write (\*PEOPLE, $nick, "$familyname, $othernames", $sex, $homepage, $tree, $comment, $link);

    close (PEOPLE);

    Log::log ($user, $tree, "Added person $nick - $familyname, $othernames");

    $init = 0;

    return "";
}


sub areSame {

    my $tree = $_[0];
    my $n1 = $_[1];
    my $n2 = $_[2];

    return 1 if ($n1 eq $n2);

    my $an1 = LinksDB::fullyResolve ($tree, $n1);
    my $an2 = LinksDB::fullyResolve ($tree, $n2);
    
    return ($an1 eq $an2);
}


sub initialise {

    my $_tree = $_[0];

    open (PEOPLE, "<./trees/$_tree/people.idx");

    my @fields = FTFile::read (\*PEOPLE);
    my $done = 0;

    while (!$done) {
        (my $k, my $d, my $s, my $hp, my $t, my $l) = @fields;

        if ($k =~ /\w+/) {
            if ($_tree eq $t) {
                $peoplelist{$k} = join ('|', @fields);
            }

            @fields = FTFile::read (\*PEOPLE);
        } else {
            $done = 1;
        }
    }

    close (PEOPLE);


    open (LINKS, "<./trees/$_tree/links.idx");

    @fields = FTFile::read (\*LINKS);
    $done = 0;

    while (!$done) {
        (my $tt, my $tn, my $ot, my $on) = @fields;

        if ($tn =~ /\w+/) {
            if ($tt eq $_tree) {
                my @el = RemoteDB::personDetails ($ot, $on);

                $peoplelist{$tn} = join ('|', @el);
            }

            @fields = FTFile::read (\*LINKS);
        } else {
            $done = 1;
        }
    }
        
    close (LINKS);

    $loadedtree = $_tree;
    $init = 1;
}


sub isKnown {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];

    if (! $init || $tree ne $loadedtree) {
        initialise ($tree);
    }

    return ($peoplelist{$nick} =~ /.+/);
}


sub replacePerson {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];
    my $familyname = $_[3];
    my $othernames = $_[4];
    my $sex = $_[5];
    my $homepage = $_[6];
    my $comment = $_[7];
    my $link = $_[8];

    my $set = 0;

    open (PEOPLE, "<./trees/$tree/people.idx");
    open (NEWPEOPLE, ">./trees/$tree/people.tmp.idx");

    PERSON: while (<PEOPLE>) {
        ($n) = split (/\|/);
        if (!$set && $n eq $nick) {
            $set = 1;
            FTFile::write (\*NEWPEOPLE, $nick, "$familyname, $othernames", $sex, $homepage, $tree, $comment, $link);
            next PERSON;
        } else {
            print NEWPEOPLE $_;
        }
    }

    if (!$set) {
        FTFile::write (\*NEWPEOPLE, $nick, "$familyname,$othernames", $sex, $homepage, $tree, $comment, $link);
    }

    close (NEWPEOPLE);
    close (PEOPLE);

    unlink "./trees/$tree/people.idx";
    rename "./trees/$tree/people.tmp.idx", "./trees/$tree/people.idx";

    Log::log ($user, $tree, "Modified person $nick");
}


sub fullName {

    my $user = $_[0];
    my $tree = $_[1];
    my $nick = $_[2];

    $nick = LinksDB::resolve ($tree, $nick);

    if ($nick =~ /\w+\.\w+/) {
        (my $n, my $name) = RemoteDB::personDetails ($tree, $nick);
        return $name;
    }

    if (! $init || $tree ne $loadedtree) {
        initialise ($tree);
    }
    (my $n, my $name) = split (/\|/, $peoplelist{$nick});

    return $name;
}


sub deletePerson {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_key=$_[2];

    open (PEOPLE, "<./trees/$_tree/people.idx");
    open (NEWPEOPLE, ">./trees/$_tree/people.tmp.idx");
    while (<PEOPLE>) {
        (my $k) = split (/\|/);
        if ($k ne $_key) {
            print NEWPEOPLE "$_";
        }
    }
    close (NEWPEOPLE);
    close (PEOPLE);

    unlink "./trees/$_tree/people.idx";
    rename "./trees/$_tree/people.tmp.idx", "./trees/$_tree/people.idx";

    if ($init) {
        delete $peoplelist{$_key};
    }

    Log::log ($user, $tree, "Deleted person $nick");
}


sub qualify {

    my $_tree=$_[0];
    my $_nick=$_[1];

    if ($_nick =~ /\w+\.\w+/) {
        return $_nick;
    }

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree);
    }

    (my $k, my $d, my $s, my $hp, my $t) = split (/\|/, $peoplelist{$_nick});

    if ($t eq $_tree) {
        return $_nick;
    } else {
        return "$t.$k";
    }
}


sub personDetails {

    my $_user=$_[0];
    my $_tree=$_[1];
    my $_nick=$_[2];

    $_nick = LinksDB::resolve ($_tree, $_nick);

    if ($_nick =~ /\w+\.\w+/) {
        return RemoteDB::personDetails ($_tree, $_nick);
    }

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree);
    }

    return split (/\|/, $peoplelist{$_nick});
}


sub allPeopleDetails {

    my $_user=$_[0];
    my $_tree=$_[1];

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree);
    }

    my @pl;

    foreach my $k (keys %peoplelist) {
        my $v = $peoplelist{$k};
        my @va = split (/\|/, $v);
#        unshift (@va, ($k));
        push @pl, [ @va ];
    }
    return @pl;
}


sub allPeople {

    my $_user=$_[0];
    my $_tree=$_[1];

    if (! $init || $_tree ne $loadedtree) {
        initialise ($_tree);
    }

    return keys %peoplelist;
}


1
